# ui_methods.py

import subprocess
import curses


class RofiUI:
    @staticmethod
    def history_menu(history):
        options = ["Search for a new torrent"]
        options.extend([f"{item['title']}" for item in history])
        result = subprocess.run(['rofi', '-dmenu', '-i', '-p', 'Select from History'],
                                input='\n'.join(options), capture_output=True, text=True)
        selected_title = result.stdout.strip()

        if selected_title == "Search for a new torrent" or not selected_title:
            return selected_title

        # Find the selected item in history
        selected_item = next(
            (item for item in history if item['title'] == selected_title), None)
        return selected_item['magnet'] if selected_item else None

    @staticmethod
    def input(prompt):
        result = subprocess.run(['rofi', '-dmenu', '-i', '-p', prompt],
                                capture_output=True, text=True)
        return result.stdout.strip()

    @staticmethod
    def menu(options):
        result = subprocess.run(['rofi', '-dmenu', '-i', '-p', 'Select Torrent'],
                                input='\n'.join(options), capture_output=True, text=True)
        return result.stdout.strip()


class CursesUI:
    @staticmethod
    def history_menu(history):
        def menu(stdscr):
            curses.curs_set(0)
            stdscr.clear()
            options = ["Search for a new torrent"]
            options.extend([f"{item['title']}" for item in history])
            current_row = 0

            while True:
                stdscr.clear()
                h, w = stdscr.getmaxyx()
                for idx, row in enumerate(options):
                    x = 0
                    y = idx
                    if y >= h:
                        break  # Avoid writing outside of the screen bounds
                    if idx == current_row:
                        stdscr.attron(curses.color_pair(1))
                        try:
                            stdscr.addstr(y, x, row)
                        except curses.error:
                            pass  # Handle writing out of bounds gracefully
                        stdscr.attroff(curses.color_pair(1))
                    else:
                        try:
                            stdscr.addstr(y, x, row)
                        except curses.error:
                            pass  # Handle writing out of bounds gracefully
                stdscr.refresh()

                key = stdscr.getch()
                if key == curses.KEY_UP and current_row > 0:
                    current_row -= 1
                elif key == curses.KEY_DOWN and current_row < len(options) - 1:
                    current_row += 1
                elif key in [curses.KEY_ENTER, ord('\n')]:
                    return options[current_row]

        selected_option = curses.wrapper(menu)
        if selected_option == "Search for a new torrent":
            return selected_option

        # Find the selected item in history
        selected_item = next(
            (item for item in history if item['title'] == selected_option), None)
        return selected_item['magnet'] if selected_item else None

    @staticmethod
    def input(prompt):
        def get_input(stdscr):
            curses.echo()
            stdscr.clear()
            stdscr.addstr(0, 0, prompt)
            stdscr.refresh()
            input_value = stdscr.getstr(1, 0).decode('utf-8')
            return input_value

        return curses.wrapper(get_input)

    @staticmethod
    def menu(options):
        def menu(stdscr):
            curses.curs_set(0)
            stdscr.clear()
            current_row = 0

            while True:
                stdscr.clear()
                h, w = stdscr.getmaxyx()
                for idx, row in enumerate(options):
                    x = 0
                    y = idx
                    if y >= h:
                        break  # Avoid writing outside of the screen bounds
                    if idx == current_row:
                        stdscr.attron(curses.color_pair(1))
                        try:
                            stdscr.addstr(y, x, row)
                        except curses.error:
                            pass  # Handle writing out of bounds gracefully
                        stdscr.attroff(curses.color_pair(1))
                    else:
                        try:
                            stdscr.addstr(y, x, row)
                        except curses.error:
                            pass  # Handle writing out of bounds gracefully
                stdscr.refresh()

                key = stdscr.getch()
                if key == curses.KEY_UP and current_row > 0:
                    current_row -= 1
                elif key == curses.KEY_DOWN and current_row < len(options) - 1:
                    current_row += 1
                elif key in [curses.KEY_ENTER, ord('\n')]:
                    return options[current_row]

        return curses.wrapper(menu)


def get_ui(ui_type):
    if ui_type == 'rofi':
        return RofiUI()
    elif ui_type == 'curses':
        return CursesUI()
    else:
        raise ValueError("Invalid UI type. Choose 'rofi' or 'curses'.")



import argparse


def parse_args():
    parser = argparse.ArgumentParser(description="Torrent Watcher Script")

    parser.add_argument(
        "--generate-config",
        action="store_true",
        help="Generate configuration file",
    )
    parser.add_argument(
        "--config",
        type=str,
        help="Specify a custom configuration file path",
    )
    parser.add_argument(
        "--history",
        action="store_true",
        help="Show torrent history",
    )
    parser.add_argument(
        "--ui",
        type=str,
        choices=["rofi", "curses"],
        default="curses",
        help="Choose the UI: rofi or curses",
    )
    parser.add_argument(
        "--player",
        type=str,
        choices=["mpv", "vlc"],
        default="mpv",
        help="Choose the Player: mpv or vlc",
    )

    return parser.parse_args()



import libtorrent as lt
import time
import tempfile
import os
import shutil
import mpv


class TorrentMPV:
    def __init__(self, magnet_link, percentage=10):
        self.magnet_link = magnet_link
        self.session = lt.session({
            'listen_interfaces': '0.0.0.0:6881'
        })
        self.download_directory = None
        self.handle = None
        self.player_process = None
        self.time_file = '.playback_time.txt'
        mpv_config_dir = os.path.join(os.path.expanduser("~"), ".config", "mpv")
        self.player = mpv.MPV(input_default_bindings=True, input_vo_keyboard=True, osc=True, config=True, config_dir=mpv_config_dir)
        self.player.observe_property('time-pos', self.time_observer)
        self.player.observe_property('playlist-pos', self.playlist_pos_observer)
        self.percentage = percentage / 100.0  # Convert percentage to a fraction

    def start_download(self):
        self.download_directory = tempfile.mkdtemp()
        print(f"Temporary download directory: {self.download_directory}")

        params = lt.add_torrent_params()
        params.save_path = self.download_directory
        params.storage_mode = lt.storage_mode_t.storage_mode_allocate
        params.url = self.magnet_link
        print("Adding torrent...")
        self.handle = self.session.add_torrent(params)
        print(f"Torrent handle created: {self.handle}")

        # Wait for metadata
        print("Waiting for metadata...")
        while not self.handle.has_metadata():
            time.sleep(1)
        print("Metadata acquired")

        self.handle.set_sequential_download(True)

        # Initially set the priorities for the first n% of each file
        self.set_priorities()

    def set_priorities(self, current_file_index=None):
        info = self.handle.get_torrent_info()
        if current_file_index is None:
            for file_index in range(info.num_files()):
                file = info.file_at(file_index)
                file_start_piece = file.offset // info.piece_length()
                file_end_piece = (file.offset + file.size) // info.piece_length()
                first_n_pieces = int((file_end_piece - file_start_piece) * self.percentage)
                for piece_index in range(file_start_piece, file_start_piece + first_n_pieces):
                    self.handle.piece_priority(piece_index, 7)  # Highest priority
            self.stream_video()
        else:
            print(f"Setting high priority for file index: {current_file_index}")
            for file_index in range(info.num_files()):
                file = info.file_at(file_index)
                file_start_piece = file.offset // info.piece_length()
                file_end_piece = (file.offset + file.size) // info.piece_length()
                priority = 7 if file_index == current_file_index else 1
                for piece_index in range(file_start_piece, file_end_piece):
                    self.handle.piece_priority(piece_index, priority)

    def stream_video(self):
        try:
            player_started = False

            while self.handle.status().state != lt.torrent_status.seeding:
                status = self.handle.status()
                print(f"Progress: {status.progress * 100:.2f}%")

                if status.progress >= 0.02 and not player_started:
                    self._launch_mpv()
                    player_started = True

                if self.player_process and self.player_process.poll() is not None:
                    print("MPV player exited.")
                    break

                time.sleep(5)
        except Exception as e:
            print(f"An error occurred: {e}")
        finally:
            self.cleanup()

    def time_observer(self, _name, value):
        if value is not None:
            time_str = 'Now playing at {:.2f}s'.format(value)
            self.write_time_to_file(time_str)

    def playlist_pos_observer(self, _name, value):
        if value is not None:
            print(f"Playlist position changed to: {value}")
            self.set_priorities(current_file_index=value)

    def write_time_to_file(self, time_str):
        with open(self.time_file, 'w') as file:
            file.write(time_str)

    def _launch_mpv(self):
        video_files = self._find_video_files()
        if video_files:
            print(f"Starting MPV with video files: {video_files}")

            self.player.loadfile(video_files[0])

            for video_file in video_files[1:]:
                print(f"Adding {video_file} to playlist")
                self.player.loadfile(video_file, mode='append-play')

            # Initially set the priorities for the first file
            self.set_priorities(current_file_index=0)

    def _find_video_files(self):
        video_files = []
        for file in self.handle.get_torrent_info().files():
            if file.path.endswith(('.mp4', '.mkv', '.avi', '.mov')):
                video_files.append(os.path.join(self.download_directory, file.path))
        return video_files

    def cleanup(self):
        print(f"Cleaning up download directory: {self.download_directory}")
        if self.handle:
            self.session.remove_torrent(self.handle)
        if self.download_directory and os.path.exists(self.download_directory):
            shutil.rmtree(self.download_directory)


if __name__ == '__main__':
    magnet_link = 'magnet:?xt=urn:btih:DD8255ECDC7CA55FB0BBF81323D87062DB1F6D1C&tr=udp%3A%2F%2Ftracker.bitsearch.to%3A1337%2Fannounce&tr=udp%3A%2F%2Fwww.torrent.eu.org%3A451%2Fannounce&tr=udp%3A%2F%2F9.rarbg.com%3A2920%2Fannounce&tr=udp%3A%2F%2Ftracker.0x.tf%3A6969%2Fannounce&tr=udp%3A%2F%2Ftracker.opentrackr.org%3A1337%2Fannounce&dn=%5BBitsearch.to%5D+Big+Buck+Bunny'

    streamer = TorrentMPV(magnet_link, percentage=5)  # Change percentage as needed
    streamer.start_download()



import os
import yaml
from pytorr_play import PyTorrPlay


def generate_config():
    config_dir = PyTorrPlay.CONFIG_DIR
    config_file = PyTorrPlay.CONFIG_FILE

    if not os.path.exists(config_dir):
        os.makedirs(config_dir)

    config = {
        "JACKETT_API_KEY": "your_api_key_here",
        "JACKET_PORT": 9117,
        "JACKET_ADDRESS": "127.0.0.1"
    }

    with open(config_file, 'w') as f:
        yaml.dump(config, f)

    print(f"Configuration file generated at {config_file}")


def load_config():
    config_file = PyTorrPlay.CONFIG_FILE
    if os.path.exists(config_file):
        with open(config_file, 'r') as f:
            return yaml.load(f, Loader=yaml.SafeLoader)
    else:
        print(f"Configuration file not found at {config_file}. Run with --generate-config to create one.")
        exit(1)


def load_config_from_path(config_path):
    if os.path.exists(config_path):
        with open(config_path, 'r') as f:
            return yaml.load(f, Loader=yaml.SafeLoader)
    else:
        print(f"Configuration file not found at {config_path}. Run with --generate-config to create one.")
        exit(1)



import requests
import platform
from plyer import notification
import xml.etree.ElementTree as ET
from fuzzywuzzy import fuzz
from datetime import datetime
import subprocess
import os
import json
from tabulate import tabulate
from ui_methods import get_ui
from streamer import TorrentMPV


class PyTorrPlay:
    CONFIG_DIR = os.path.join(os.path.expanduser(
        "~"), ".config", "torrent_watcher")
    CONFIG_FILE = os.path.join(CONFIG_DIR, "config.yaml")
    HISTORY_DIR = os.path.join(os.path.expanduser("~"), ".torrent_watcher")
    HISTORY_FILE = os.path.join(HISTORY_DIR, "history.json")
    HISTORY_LIMIT = 16

    def __init__(self, config, ui_type, player):
        self.config = config
        self.JACKETT_API_KEY = config.get("JACKETT_API_KEY", "")
        self.JACKET_PORT = config.get("JACKET_PORT", 9117)
        self.JACKET_ADDRESS = config.get("JACKET_ADDRESS", "127.0.0.1")
        self.BASE_URL = f"http://{self.JACKET_ADDRESS}:{self.JACKET_PORT}/api/v2.0/indexers/all/results/torznab?apikey={self.JACKETT_API_KEY}&q="
        self.PLAYER = player

        if not os.path.exists(self.HISTORY_DIR):
            os.makedirs(self.HISTORY_DIR)
        self.history = self.load_history()

        # Initialize UI
        self.ui = get_ui(ui_type)

    def notify(self, message):
        title = "Notification"
        system = platform.system()

        if system in ['Linux', 'Darwin', 'Windows']:
            notification.notify(
                title=title,
                message=message,
                app_name='PyTorr-Play',
                timeout=10  # duration in seconds
            )
        else:
            print("Unsupported OS")

    def load_history(self):
        if os.path.exists(self.HISTORY_FILE):
            with open(self.HISTORY_FILE, 'r') as f:
                return json.load(f)
        return []

    def save_history(self):
        with open(self.HISTORY_FILE, 'w') as f:
            json.dump(self.history, f)

    def add_to_history(self, title, magnet):
        timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.history = [
            item for item in self.history if item['magnet'] != magnet]
        self.history.insert(
            0, {'title': title, 'magnet': magnet, 'timestamp': timestamp})
        if len(self.history) > self.HISTORY_LIMIT:
            self.history.pop()
        self.save_history()

    def show_history(self):
        if self.history:
            headers = ["Title", "Magnet", "Timestamp"]
            table = []
            for item in self.history:
                shortened_magnet = item['magnet'][:10] + \
                    "..." + item['magnet'][-10:]
                table.append(
                    [item['title'], shortened_magnet, item['timestamp']])
            print(tabulate(table, headers, tablefmt="grid"))
        else:
            print("No history available.")

    def fetch_results(self, query):
        self.notify("Fetching tracker data...")
        url = self.BASE_URL + query
        response = requests.get(url)
        response.raise_for_status()
        self.notify("Finished fetching tracker data.")
        return response.content

    def parse_xml(self, xml_data):
        ns = {'torznab': 'http://torznab.com/schemas/2015/feed'}
        root = ET.fromstring(xml_data)
        items = []
        for item in root.findall(".//item"):
            title = item.find("title").text
            seeders = int(
                item.find("torznab:attr[@name='seeders']", ns).attrib['value'])
            link = item.find("link").text
            magnet_element = item.find("torznab:attr[@name='magneturl']", ns)
            magnet = magnet_element.attrib['value'] if magnet_element is not None else None

            if magnet:
                items.append({'title': title, 'seeders': seeders,
                             'link': link, 'magnet': magnet})
        return items

    def sort_results(self, items, query):
        for item in items:
            item['score'] = fuzz.partial_ratio(
                item['title'].lower(), query.lower())
        sorted_items = sorted(
            items, key=lambda x: (-x['seeders'], -x['score']))
        return sorted_items

    def main(self):
        selected_option = self.ui.history_menu(self.history)
        if not selected_option:
            return

        if selected_option == "Search for a new torrent":
            query = self.ui.input("Enter your search query:")
            if not query:
                return

            xml_data = self.fetch_results(query)
            items = self.parse_xml(xml_data)
            sorted_items = self.sort_results(items, query)

            options = [
                f"{item['title']} - Seeders: {item['seeders']}" for item in sorted_items]
            selected_option = self.ui.menu(options)

            if selected_option:
                selected_item = next(item for item in sorted_items if f"{item['title']} - Seeders: {item['seeders']}" == selected_option)
        else:
            selected_item = next(
                (item for item in self.history if item['magnet']
                 == selected_option),
                None
            )

        if selected_item['magnet']:
            self.notify(f"Starting torrent download: {selected_item['title']}")
            self.add_to_history(
                selected_item['title'], selected_item['magnet'])
            if self.PLAYER == "mpv":
                player = TorrentMPV(selected_item['magnet'])
                player.start_download()
                self.notify("Starting playback with mpv...")
            else:
                subprocess.run(['webtorrent', 'download', '--playlist', f'--vlc', '-u', '100', selected_item['magnet']])
                self.notify("Starting playback with vlc...")
        else:
            print("No magnet link found for the selected torrent.")




from pytorr_play import PyTorrPlay
from config_utils import generate_config, load_config_from_path
from arg_parser import parse_args


def main():
    args = parse_args()

    if args.generate_config:
        generate_config()
    else:
        config_path = args.config if args.config else PyTorrPlay.CONFIG_FILE
        config = load_config_from_path(config_path)
        watcher = PyTorrPlay(config, args.ui, args.player)

        if args.history:
            watcher.show_history()
        else:
            watcher.main()


if __name__ == "__main__":
    main()
